import pygame, random

# konstante - boje
BLACK = (   0,   0,   0)
WHITE = ( 255, 255, 255)
RED   = ( 255,   0,   0)
BLUE  = ( 0,   0,   255)

WALL_SIZE = 10
STEP = 8

PADDLE_STEP = 10
LEFT  = 0
RIGHT = 1

WINNING_SCORE = 5

class BlockSprite(pygame.sprite.Sprite):
    
    def __init__(self, x, y, width, height, color=BLACK):
        super().__init__()
        self.image = pygame.Surface((width, height))
        self.image.fill(color)
        self.rect = self.image.get_rect()
        self.rect.topleft = (x, y)
# ---------------------------------------------------------

class Paddle(BlockSprite):
    
    def __init__(self, x, y):
        super().__init__(x, y-75, 10, 150, BLUE)  # sirina i visina reketa

    def move(self, step):
        if pygame.sprite.collide_rect(self, top) and (step < 0):  # na vrhu i ide na gore
            step = 0
        elif pygame.sprite.collide_rect(self, bottom) and (step > 0):  
            # na dnu i ide na dole
            step = 0
        self.rect.y += step

# ---------------------------------------------------------

class BallSprite(pygame.sprite.Sprite):

    def __init__(self, fnm):
        super().__init__()
        self.image = pygame.image.load(fnm).convert_alpha()
        self.rect = self.image.get_rect()
        self.rect.center = [scrWidth/2, scrHeight/2]
                       # pocetna pozicija lopte u sredini prozora
        self.xStep, self.yStep = self.randomSteps()
                       # velicina koraka i pravac duz svake ose

    def update(self):
        global scoreLeft, scoreRight
        if pygame.sprite.collide_rect(self, leftPaddle) and (self.xStep < 0):  
            # udari levi reket i ide levo
            hitPaddle.play() 
            self.xStep = -self.xStep    # promena pravca

        elif pygame.sprite.collide_rect(self, rightPaddle) and (self.xStep > 0):  
            # udari desni reket i ide desno
            hitPaddle.play() 
            self.xStep = -self.xStep    # promena pravca

        if pygame.sprite.spritecollideany(self, horizWalls):
            # promena pravca po y
            hitWall.play() 
            self.yStep = -self.yStep

        if pygame.sprite.spritecollideany(self, vertWalls):
            # lopta je stigla do leve ili desne strane
            if pygame.sprite.collide_rect(self, right):
                scoreLeft += 1
            else:   # leva strana
                scoreRight += 1
            scorePoint.play() 

            # resetujte loptu
            self.rect.center = (scrWidth/2, scrHeight/2)
            self.xStep, self.yStep = self.randomSteps()

        self.rect.x += self.xStep   # horizontalno pomeranje lopte
        self.rect.y += self.yStep   # vertikalno pomeranje lopte

    def randomSteps(self):
        # kreiranje nasumicnog +/- para koraka
        x = STEP
        if random.random() > 0.5:
            x = -x
        y = STEP
        if random.random() > 0.5:
            y = -y
        return [x,y]
# -----------------------------------

def initMusic():
    global hitPaddle, hitWall, scorePoint, bye

    # ucitavanje muzike i postavljanje jacine zvuka
    pygame.mixer.music.load("bg_music.mp3") 
    pygame.mixer.music.set_volume(0.3)
    pygame.mixer.music.play(-1)   # pustaj neprekidno

    # ucitavanje zvucnih efekata i podesavanje njihove jacine
    hitPaddle = pygame.mixer.Sound("hitPaddle.wav")         
    hitPaddle.set_volume(0.4)    
                            
    hitWall = pygame.mixer.Sound("hitWall.wav")      
    hitWall.set_volume(0.4)   
                        
    scorePoint = pygame.mixer.Sound("scorePoint.wav")    
    scorePoint.set_volume(0.2)  
                        
    bye = pygame.mixer.Sound("gameOver.wav")          
    bye.set_volume(0.6)                                

def centerImage(screen, im):
    x = (scrWidth - im.get_width())/2
    y = (scrHeight - im.get_height())/2
    screen.blit(im, (x,y))
    
# ---------- glavni deo programa -------------

pygame.init()
screen = pygame.display.set_mode([640,480])
screen.fill(WHITE)
pygame.display.set_caption("Pong")

scrWidth, scrHeight = screen.get_size()

initMusic()

# kreiranje sprajtova za zidove
top    = BlockSprite(0, 0, scrWidth, WALL_SIZE)
bottom = BlockSprite(0, scrHeight-WALL_SIZE, scrWidth, WALL_SIZE)
left   = BlockSprite(0, 0, WALL_SIZE, scrHeight)
right  = BlockSprite(scrWidth-WALL_SIZE, 0, WALL_SIZE, scrHeight)

horizWalls = pygame.sprite.Group(top, bottom)
vertWalls = pygame.sprite.Group(left, right)

# kreiranje reketa
leftPaddle = Paddle(50, scrHeight/2)
rightPaddle = Paddle(scrWidth-50, scrHeight/2)

ball = BallSprite('smallBall.png')

sprites = pygame.sprite.OrderedUpdates(top, bottom, left, right, 
                               leftPaddle, rightPaddle, ball)

leftStep = 0
rightStep = 0

scoreLeft = 0
scoreRight = 0
winMsg = ""
gameOver = False

font = pygame.font.Font(None, 72)

clock = pygame.time.Clock()

running = True    
while running:
    clock.tick(30)

    # obrada dogadjaja
    for event in pygame.event.get():
        if event.type == pygame.QUIT: 
            running = False
    
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_q:   # levi reket
                leftStep = -PADDLE_STEP    # gore
            elif event.key == pygame.K_s:
                leftStep = PADDLE_STEP     # dole

            if event.key == pygame.K_p:   # desni reket
                rightStep = -PADDLE_STEP   # gore
            elif event.key == pygame.K_l:
                rightStep = PADDLE_STEP    # dole

        elif event.type == pygame.KEYUP: 
            if event.key == pygame.K_q or event.key == pygame.K_s:   # levi reket
                leftStep = 0
            if event.key == pygame.K_p or event.key == pygame.K_l:   # desni reket
                rightStep = 0


    # azuriraj igru
    if not gameOver:
        leftPaddle.move(leftStep)
        rightPaddle.move(rightStep)
        ball.update()

        if scoreLeft >= WINNING_SCORE:
            winMsg = "Pobednik je levi!"
            gameOver = True
            bye.play()
        elif scoreRight >= WINNING_SCORE:
            winMsg = "Pobednik je desni!"
            gameOver = True
            bye.play()

    screen.fill(WHITE)                       
    sprites.draw(screen)

    screen.blit( font.render(str(scoreLeft) + ":" + 
                             str(scoreRight), True, RED), [20, 20])

    if gameOver:
        centerImage(screen, font.render(winMsg, True, RED))

    pygame.display.update()

pygame.quit()